@@ -1,4 +1,5 @@
 import os
+import re
 import sys
 import cgi
 import shutil
@@ -9,6 +10,7 @@ import functools
 from subprocess import check_call
 import pkg_resources
 import glob
+import optparse
 
 try:
     import wheel as _; _
@@ -76,6 +78,8 @@ def file_to_package(file, basedir=None):
         ('foo', '1.2.3-rc1.tar.gz')
         >>> file_to_package("foo-bar-1.2.tgz")
         ('foo-bar', '1.2.tgz')
+        >>> file_to_package("kafka-quixey-0.8.1-1.tar.gz")
+        ('kafka-quixey', '0.8.1-1.tar.gz')
         >>> file_to_package("foo-bar-1.2-py27-none-any.whl")
         ('foo-bar', '1.2-py27-none-any.whl')
         >>> file_to_package("Cython-0.17.2-cp26-none-linux_x86_64.whl")
@@ -91,7 +95,8 @@ def file_to_package(file, basedir=None):
         split = [bits[0], "-".join(bits[1:])]
         to_safe_name = lambda x: x
     else:
-        split = file.rsplit("-", 1)
+        match = re.search(r"(?P<pkg>.*?)-(?P<rest>\d+.*)", file)
+        split = match.group("pkg"), match.group("rest")
         to_safe_name = pkg_resources.safe_name
 
     if len(split) != 2 or not split[1]:
@@ -135,11 +140,45 @@ def pip_run_command(pip_args):
                        %(res, pip_args))
 
 
-def dir2pi(argv=sys.argv):
-    if len(argv) != 2:
-        print(dedent("""
-            usage: dir2pi PACKAGE_DIR
+OS_HAS_SYMLINK = hasattr(os, "symlink")
 
+
+class Pip2PiOptionParser(optparse.OptionParser):
+    def format_description(self, formatter):
+        # Parent implementation reformats all the hard line breaks
+        return self.get_description() + "\n"
+
+    def add_symlink_option(self, use_symlink):
+        self.add_option('-s', '--symlink', dest="use_symlink",
+                        default=use_symlink, action="store_true",
+                        help="Use symlinks in PACKAGE_DIR/simple/ "
+                             "rather than copying files. Default: %default")
+        self.add_option('-S', '--no-symlink', dest="use_symlink",
+                        action="store_false")
+
+    def _process_args(self, largs, rargs, values):
+        """
+        An unknown option pass-through implementation of OptionParser.
+
+        When unknown arguments are encountered, bundle with largs and try again,
+        until rargs is depleted.  
+
+        sys.exit(status) will still be called if a known argument is passed
+        incorrectly (e.g. missing arguments or bad argument types, etc.)        
+
+        From http://stackoverflow.com/a/9307174/6364
+        """
+        while rargs:
+            try:
+                optparse.OptionParser._process_args(self, largs, rargs, values)
+            except (optparse.BadOptionError, optparse.AmbiguousOptionError) as e:
+                largs.append(e.opt_str)
+
+
+def dir2pi(argv=sys.argv, use_symlink=OS_HAS_SYMLINK):
+    parser = Pip2PiOptionParser(
+        usage="usage: %prog PACKAGE_DIR",
+        description=dedent("""
             Creates the directory PACKAGE_DIR/simple/ and populates it with the
             directory structure required to use with pip's --index-url.
 
@@ -148,7 +187,7 @@ def dir2pi(argv=sys.argv):
             'foo-bar-1.3rc1.bz2').
 
             This makes the most sense if PACKAGE_DIR is somewhere inside a
-            webserver's inside htdocs directory.
+            webserver's htdocs directory.
 
             For example:
 
@@ -163,7 +202,13 @@ def dir2pi(argv=sys.argv):
                 packages/simple/foo/index.html
                 packages/simple/foo/foo-1.2.tar.gz
         """))
-        return 1
+    parser.add_symlink_option(use_symlink)
+
+    option, argv = parser.parse_args(argv)
+    if len(argv) != 2:
+        parser.print_help()
+        parser.exit()
+
     pkgdir = argv[1]
     if not os.path.isdir(pkgdir):
         raise ValueError("no such directory: %r" %(pkgdir, ))
@@ -188,7 +233,7 @@ def dir2pi(argv=sys.argv):
         pkg_new_basename = "-".join([pkg_name, pkg_rest])
         symlink_target = os.path.join(pkg_dir, pkg_new_basename)
         symlink_source = os.path.join("../../", pkg_basename)
-        if hasattr(os, "symlink"):
+        if option.use_symlink and OS_HAS_SYMLINK:
             os.symlink(symlink_source, symlink_target)
         else:
             shutil.copy2(pkg_filepath, symlink_target)
@@ -212,11 +257,9 @@ def globall(globs):
 @maintain_cwd
 def pip2tgz(argv=sys.argv):
     glob_exts = ['*.whl', '*.tgz', '*.gz']
-
-    if len(argv) < 3:
-        print(dedent("""
-            usage: pip2tgz OUTPUT_DIRECTORY PACKAGE_NAME ...
-
+    parser = Pip2PiOptionParser(
+        usage="usage: %prog OUTPUT_DIRECTORY PACKAGE_NAME ...",
+        description=dedent("""
             Where PACKAGE_NAMES are any names accepted by pip (ex, `foo`,
             `foo==1.2`, `-r requirements.txt`).
 
@@ -227,7 +270,11 @@ def pip2tgz(argv=sys.argv):
 
                 $ pip2tgz /var/www/packages/ -r requirements.txt foo==1.2 baz/
         """))
-        return 1
+
+    option, argv = parser.parse_args(argv)
+    if len(argv) < 3:
+        parser.print_help()
+        parser.exit()
 
     outdir = os.path.abspath(argv[1])
     if not os.path.exists(outdir):
@@ -299,10 +346,9 @@ def handle_new_wheels(outdir, new_wheels):
         ])
 
 def pip2pi(argv=sys.argv):
-    if len(argv) < 3:
-        print(dedent("""
-            usage: pip2pi TARGET PACKAGE_NAME ...
-
+    parser = Pip2PiOptionParser(
+        usage="usage: %prog TARGET PACKAGE_NAME ...",
+        description=dedent("""
             Combines pip2tgz and dir2pi, adding PACKAGE_NAME to package index
             TARGET.
 
@@ -318,7 +364,12 @@ def pip2pi(argv=sys.argv):
 
                 $ pip2pi ~/Sites/packages/ foo==1.2
         """))
-        return 1
+    parser.add_symlink_option(OS_HAS_SYMLINK)
+
+    option, argv = parser.parse_args(argv)
+    if len(argv) < 3:
+        parser.print_help()
+        parser.exit()
 
     target = argv[1]
     pip_packages = argv[2:]
@@ -335,7 +386,7 @@ def pip2pi(argv=sys.argv):
         print("pip2tgz returned an error; aborting.")
         return res
 
-    res = dir2pi([argv[0], working_dir])
+    res = dir2pi([argv[0], working_dir], use_symlink=option.use_symlink)
     if res:
         print("dir2pi returned an error; aborting.")
         return res
